from os import (path)
import sys
from io import StringIO
from collections import namedtuple
from argparse import ArgumentParser
from configparser import ConfigParser

from botocore.session import Session
from .configfilewriter import ConfigFileWriter

aws_config_path = path.expanduser(
    Session().get_config_variable('config_file'))
    
aws_credentials_path = path.expanduser(
    Session().get_config_variable('credentials_file'))


def split_key_value(key_value, separator):
    key_value_split = key_value.split(separator, 1)
    if len(key_value_split) != 2:
        raise ValueError("Invalid key value '" + key_value + "' expected format '<key>" +
                         separator + "<value>', but was '" + key_value + "'")
    return namedtuple('KeyValue', ['key', 'value'])(key=key_value_split[0], value=key_value_split[1])


def profile_config_section(profile_name):
    return "profile " + profile_name if profile_name != "default" else "default"


def profile_update(config_path, profile_section, config, merge=False):

    if not merge:
        profile_delete(config_path, profile_section, clear=True)

    if config:
        # add empty line as profile separator
        current_config = ConfigParser()
        current_config.read(config_path)
        if current_config.sections() and profile_section not in current_config.sections():
            with open(config_path, 'a') as config_file:
                config_file.write('\n')

        ConfigFileWriter().update_config({
            **config,
            '__section__': profile_section
        }, config_path)


def profile_delete(config_path, profile_section, clear=False):
    if not path.exists(config_path):
        return
    
    with open(config_path) as config_file:
        config_string = config_file.read()

    with open(config_path, 'w') as config_file:
        output = True
        for config_line in StringIO(config_string):
            section_match = ConfigFileWriter.SECTION_REGEX.search(config_line)
            if section_match:
                if section_match.group('header') == profile_section:
                    if clear:
                        config_file.write(config_line)
                    output = False
                else:
                    if clear and not output:
                        config_file.write('\n')
                    output = True
            if output:
                config_file.write(config_line)


def handle_help(args):
    print("""\
usage:

    list profiles:

        aws-configure list
        

    set profile options:

        aws-configure set [--profile/-p <profile_name>] [--empty/-e] [<config_options...>]

            --profile/-p <profile_name> : select profile ['default']
            <config_options>            : key=value pairs e.g. 'region=eu-central-1' 'source_profile=default'
            --empty/-e                  : empty all profile options before setting new options
            --empty-config              : empty profile config options before setting
            --empty-credentials         : empty profile credentials options before setting

    get profile options:

        aws-configure get [--profile/-p <profile_name>] [<config_options...>]

            --profile/-p <profile_name> : select profile ['default']
            <config_options>            : option key e.g. 'region' 'source_profile'

    delete profile:

        aws-configure delete [--profile/-p <profile_name>] [--config] [--credentials]

            --profile/-p <profile_name> : select profile ['default']
            --config                    : delete only profile config in '~/.aws/config'
            --credentials               : delete only profile credentials in '~/.aws/credentials'

    print help

        aws-configure help
""")


def handle_list_profiles(args):
    profile_map = Session().full_config['profiles']
    for profile_name, profile in profile_map.items():
        print(profile_name)


def handle_delete_profile(args):
    profile_name = args.profile_name or 'default'
    delete_all= not args.delete_config and not args.delete_credentials
    delete_config = delete_all or args.delete_config
    delete_credentials = delete_all or args.delete_credentials

    if delete_config:
        print("delete profile config")
        profile_delete(aws_config_path, profile_config_section(profile_name))

    if delete_credentials:
        print("delete profile credentials")
        profile_delete(aws_credentials_path, profile_name)


def handle_set_profile(args):
    profile_name = args.profile_name or 'default'
    profile_config = {}
    profile_credentials = {}
    for option in args.profile_options:
        option = split_key_value(option, '=')
        if option.key not in ['aws_access_key_id', 'aws_secret_access_key', 'aws_session_token']:
            profile_config[option.key] = option.value
        else:
            profile_credentials[option.key] = option.value

    if args.empty_config or args.empty_all:
        print("empty '" + profile_name + "' profile config options")
        profile_update(aws_config_path, profile_config_section(profile_name),{})

    if profile_config:
        print("set '" + profile_name + "' profile config options")
        profile_update(aws_config_path, profile_config_section(profile_name),
                       profile_config, merge=True)

    if args.empty_credentials or args.empty_all:
        print("empty '" + profile_name + "' profile credentials")
        profile_update(aws_credentials_path, profile_name,{})

    if profile_credentials:
        print("set '" + profile_name + "' profile credentials")
        profile_update(aws_credentials_path, profile_name,
                       profile_credentials, merge=True)


def handle_get_profile(args):
    profile_name = args.profile_name or 'default'
    aws_config_section=profile_config_section(profile_name)
    aws_config = ConfigParser()
    aws_config.read(aws_config_path)
    if args.profile_options:
        for option in args.profile_options:
            print(aws_config.get(aws_config_section, option))
    else:
        for option in aws_config.options(aws_config_section):
            print(option + " = " + aws_config.get(aws_config_section, option))
            
            
def main():
    parser = ArgumentParser(add_help=False)

    parser_command = parser.add_subparsers(title='commands',dest='command')
    parser_command.required = True

    parser_command_list = parser_command.add_parser('list', help="List profiles")
    parser_command_list.set_defaults(func=handle_list_profiles)

    parser_command_set = parser_command.add_parser('set', help="Set profile options")
    parser_command_set.add_argument('-p', '--profile', dest='profile_name', help='Profile name')
    parser_command_set.add_argument('-e', '--empty', action='store_true', dest='empty_all', help='Empty all profile options before setting new options')
    parser_command_set.add_argument('--empty-config', action='store_true', dest='empty_config', help='Empty profile config options before setting new options')
    parser_command_set.add_argument('--empty-credentials', action='store_true', dest='empty_credentials', help='Empty profile credentials options before setting new options')
    parser_command_set.add_argument(dest='profile_options', nargs='*', help='Profile options')
    parser_command_set.set_defaults(func=handle_set_profile)
    
    parser_command_get = parser_command.add_parser('get', help="Get profile options")
    parser_command_get.add_argument('-p', '--profile', dest='profile_name', help='Profile name')
    parser_command_get.add_argument(dest='profile_options', nargs='*', help='Profile options')
    parser_command_get.set_defaults(func=handle_get_profile)

    parser_command_delete = parser_command.add_parser('delete', help='Delete profile')
    parser_command_delete.add_argument('-p', '--profile', dest='profile_name', help='Profile name')
    parser_command_delete.add_argument('--config', action='store_true', dest='delete_config', help='Delete only profile config')
    parser_command_delete.add_argument('--credentials', action='store_true', dest='delete_credentials', help='Delete only profile credentials')
    parser_command_delete.set_defaults(func=handle_delete_profile)
    
    parser_command_help = parser_command.add_parser('help', help="Print help")
    parser_command_help.set_defaults(func=handle_help)

    args = parser.parse_args()        
    args.func(args)

if __name__ == "__main__":
    main()



#
# Origin: https://github.com/aws/aws-cli/blob/master/awscli/customizations/configure/writer.py
#
# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import re

class SectionNotFoundError(Exception):
    pass

class ConfigFileWriter(object):
    SECTION_REGEX = re.compile(r'\[(?P<header>[^]]+)\]')
    OPTION_REGEX = re.compile(
        r'(?P<option>[^:=][^:=]*)'
        r'\s*(?P<vi>[:=])\s*'
        r'(?P<value>.*)$'
    )

    def update_config(self, new_values, config_filename):
        """Update config file with new values.
        This method will update a section in a config file with
        new key value pairs.
        This method provides a few conveniences:
        * If the ``config_filename`` does not exist, it will
          be created.  Any parent directories will also be created
          if necessary.
        * If the section to update does not exist, it will be created.
        * Any existing lines that are specified by ``new_values``
          **will not be touched**.  This ensures that commented out
          values are left unaltered.
        :type new_values: dict
        :param new_values: The values to update.  There is a special
            key ``__section__``, that specifies what section in the INI
            file to update.  If this key is not present, then the
            ``default`` section will be updated with the new values.
        :type config_filename: str
        :param config_filename: The config filename where values will be
            written.
        """
        section_name = new_values.pop('__section__', 'default')
        if not os.path.isfile(config_filename):
            self._create_file(config_filename)
            self._write_new_section(section_name, new_values, config_filename)
            return
        with open(config_filename, 'r') as f:
            contents = f.readlines()
        # We can only update a single section at a time so we first need
        # to find the section in question
        try:
            self._update_section_contents(contents, section_name, new_values)
            with open(config_filename, 'w') as f:
                f.write(''.join(contents))
        except SectionNotFoundError:
            self._write_new_section(section_name, new_values, config_filename)

    def _create_file(self, config_filename):
        # Create the file as well as the parent dir if needed.
        dirname = os.path.split(config_filename)[0]
        if not os.path.isdir(dirname):
            os.makedirs(dirname)
        with os.fdopen(os.open(config_filename,
                               os.O_WRONLY | os.O_CREAT, 0o600), 'w'):
            pass

    def _write_new_section(self, section_name, new_values, config_filename):
        with open(config_filename, 'a') as f:
            f.write('[%s]\n' % section_name)
            contents = []
            self._insert_new_values(line_number=0,
                                    contents=contents,
                                    new_values=new_values)
            f.write(''.join(contents))

    def _find_section_start(self, contents, section_name):
        for i in range(len(contents)):
            line = contents[i]
            if line.strip().startswith(('#', ';')):
                # This is a comment, so we can safely ignore this line.
                continue
            match = self.SECTION_REGEX.search(line)
            if match is not None and self._matches_section(match,
                                                           section_name):
                return i
        raise SectionNotFoundError(section_name)

    def _update_section_contents(self, contents, section_name, new_values):
        # First, find the line where the section_name is defined.
        # This will be the value of i.
        new_values = new_values.copy()
        # ``contents`` is a list of file line contents.
        section_start_line_num = self._find_section_start(contents,
                                                          section_name)
        # If we get here, then we've found the section.  We now need
        # to figure out if we're updating a value or adding a new value.
        # There's 2 cases.  Either we're setting a normal scalar value
        # of, we're setting a nested value.
        last_matching_line = section_start_line_num
        j = last_matching_line + 1
        while j < len(contents):
            line = contents[j]
            if self.SECTION_REGEX.search(line) is not None:
                # We've hit a new section which means the config key is
                # not in the section.  We need to add it here.
                self._insert_new_values(line_number=last_matching_line,
                                        contents=contents,
                                        new_values=new_values)
                return
            match = self.OPTION_REGEX.search(line)
            if match is not None:
                last_matching_line = j
                key_name = match.group(1).strip()
                if key_name in new_values:
                    # We've found the line that defines the option name.
                    # if the value is not a dict, then we can write the line
                    # out now.
                    if not isinstance(new_values[key_name], dict):
                        option_value = new_values[key_name]
                        new_line = '%s = %s\n' % (key_name, option_value)
                        contents[j] = new_line
                        del new_values[key_name]
                    else:
                        j = self._update_subattributes(
                            j, contents, new_values[key_name],
                            len(match.group(1)) - len(match.group(1).lstrip()))
                        return
            j += 1

        if new_values:
            if not contents[-1].endswith('\n'):
                contents.append('\n')
            self._insert_new_values(line_number=last_matching_line + 1,
                                    contents=contents,
                                    new_values=new_values)

    def _update_subattributes(self, index, contents, values, starting_indent):
        index += 1
        for i in range(index, len(contents)):
            line = contents[i]
            match = self.OPTION_REGEX.search(line)
            if match is not None:
                current_indent = len(
                    match.group(1)) - len(match.group(1).lstrip())
                key_name = match.group(1).strip()
                if key_name in values:
                    option_value = values[key_name]
                    new_line = '%s%s = %s\n' % (' ' * current_indent,
                                                key_name, option_value)
                    contents[i] = new_line
                    del values[key_name]
            if starting_indent == current_indent or \
                    self.SECTION_REGEX.search(line) is not None:
                # We've arrived at the starting indent level so we can just
                # write out all the values now.
                self._insert_new_values(i - 1, contents, values, '    ')
                break
        else:
            if starting_indent != current_indent:
                # The option is the last option in the file
                self._insert_new_values(i, contents, values, '    ')
        return i

    def _insert_new_values(self, line_number, contents, new_values, indent=''):
        new_contents = []
        for key, value in list(new_values.items()):
            if isinstance(value, dict):
                subindent = indent + '    '
                new_contents.append('%s%s =\n' % (indent, key))
                for subkey, subval in list(value.items()):
                    new_contents.append('%s%s = %s\n' % (subindent, subkey,
                                                         subval))
            else:
                new_contents.append('%s%s = %s\n' % (indent, key, value))
            del new_values[key]
        contents.insert(line_number + 1, ''.join(new_contents))

    def _matches_section(self, match, section_name):
        parts = section_name.split(' ')
        unquoted_match = match.group(0) == '[%s]' % section_name
        if len(parts) > 1:
            quoted_match = match.group(0) == '[%s "%s"]' % (
                parts[0], ' '.join(parts[1:]))
            return unquoted_match or quoted_match
        return unquoted_match


import setuptools
import os
__directory__=os.path.dirname(os.path.realpath(__file__))
os.chdir(__directory__)

def read_text(file_name):
    with open(file_name) as file:
        return file.read()

setuptools.setup(
    name='aws-configure',
    version='2.1.8',
    author="Bengt Brodersen",
    author_email="me@qoomon.me",
    description="A CLI to configure AWS named profiles in ~/.aws/config and ~/.aws/credentials files",
    long_description=read_text("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/qoomon/aws-configure",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: System :: Systems Administration",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators"
    ],
    install_requires=[
        'awscli',
        'botocore'
    ],
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'aws-configure = aws_configure.cli:main',
        ]
    }
)
